//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "Mail2NewsDlgs.h"
#include "AskUpdateDlgs.h"
#include "httpmods.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMail2newsDlg *Mail2newsDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageMail2newsTxt(TForm *Owner, const AnsiString &FileName)
{
   Mail2newsDlg = new TMail2newsDlg(Owner, FileName);
   Mail2newsDlg->ShowModal();
   bool ok = Mail2newsDlg->ModalResult == mrOk;
   delete Mail2newsDlg;
   return ok;
}

//---------------------------------------------------------------------------
// TListManDlg
//---------------------------------------------------------------------------

__fastcall TMail2newsDlg::TMail2newsDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TMail2newsDlg::TMail2newsDlg(TComponent* Owner, const AnsiString &FileName)
        : TForm(Owner)
{
   OrgPage = new TStringList();
   filename = FileName;

   if (FileExists(filename))
   {
      Editor->Lines->LoadFromFile(filename);
   }

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, NULL, Editor->Font);
   Editor->Color = color;

   Editor->Modified = false;

   ProgressBar->Top = 6;
}
//---------------------------------------------------------------------------

__fastcall TMail2newsDlg::~TMail2newsDlg()
{
   delete OrgPage;
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Working)
   {
      MessageBeep(MB_OK);
      CanClose = false;
      return;
   }

   if (Editor->Modified)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this,
            "Your list is modified. Save Your changes?", MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      Editor->Lines->SaveToFile(filename);
   }
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::SetToolbarButtons()
{
   PasteBtn->Enabled = Clipboard()->HasFormat(CF_TEXT);
   bool hassel = Editor->SelLength > 0;
   CopyBtn->Enabled = hassel;
   CutBtn->Enabled = hassel;
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::PopupMenuPopup(TObject *Sender)
{
   PopPasteItem->Enabled = Clipboard()->HasFormat(CF_TEXT);
   bool hassel = Editor->SelLength > 0;
   PopCopyItem->Enabled = hassel;
   PopCutItem->Enabled = hassel;
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::EditorSelectionChange(TObject *Sender)
{
   SetToolbarButtons();
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::FormResize(TObject *Sender)
{
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::FormPaint(TObject *Sender)
{
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetMail2newsURL()
{
   TStringList *urls = new TStringList();
   if (FileExists(AppDir() + "urls.txt"))
   {
      urls->LoadFromFile(AppDir() + "urls.txt");
   }

   bool inQS = false;
   AnsiString URL, line;

   for (int i = 0; i < urls->Count; i++)
   {
      AnsiString url = urls->Strings[i].Trim().LowerCase();
      if (url.Pos("[quicksilver]") == 1)
      {
         inQS = true;
         continue;
      }

      if (inQS)
      {
         if (url.Pos("mail2news.txt ") == 1)
         {
            URL = GetTokSpc(1, url);
            break;
         }

         if (url.Pos("[") == 1)
         {
            break;
         }
      }
   }

   delete urls;
   return URL;
}
//---------------------------------------------------------------------------

bool __fastcall TMail2newsDlg::Refresh()
{
   TStringList *proxy = new TStringList();

   if (!AskUpdateMail2newsGateways(this, proxy))
   {
      delete proxy;
      return false;
   }

   Cancel = false;
   AnsiString URL = GetMail2newsURL();

   if (URL.IsEmpty())
   {
      MsgBox(this,
        "I can't find the URL for nymserver.asc! It should be listed in the    \n"
        "QuickSilver section of urls.txt. You can check this by opening QSstats    \n"
        "and click 'Tools|URL Manager'. QuickSilver is at the bottom.",
      MT_ERROR, MB_OK);
      delete proxy;
      return false;
   }

   TStringList *page = new TStringList();

   CutBtn->Enabled = false;
   CopyBtn->Enabled = false;
   PasteBtn->Enabled = false;
   HttpBtn->Enabled = false;
   OkBtn->Enabled = false;

   ProgressBar->Show();
   Working = true;

   TDateTime datetime = 0;
   int result = HTTPGet(this, URL, page, ProgressBar, proxy, &Cancel, 30, datetime);

   Working = false;
   ProgressBar->Hide();

   delete proxy;

   if (Cancel || result == HTTP_TIMEOUT)
   {
      MsgBox(this, "Sorry, the connection timed out. Try again later.",
         MT_INFO, MB_OK);
      HttpBtn->Enabled = true;
      OkBtn->Enabled = true;
      SetToolbarButtons();
      delete page;
      return false;
   }

   if (result != HTTP_OK)
   {
      MsgBox(this, page->Strings[0] + "    ", MT_ERROR, MB_OK);
      HttpBtn->Enabled = true;
      OkBtn->Enabled = true;
      SetToolbarButtons();
      delete page;
      return false;
   }

   TrimNulls(page);

   boguspage = false;
   if (page->Text.Pos("mail2news@") == 0)
   {
      OrgPage->Text = Editor->Lines->Text;
      OrgModified = Editor->Modified;
      OkBtn->Enabled = false;
      Editor->Lines->Clear();
      boguspage = true;
      Snooze(200);
      Editor->Lines->AddStrings(page);
      Snooze(200);

      MsgBox(this,
         "The new page doesn't look correct. I can't find any addresses starting    \n"
         "with 'mail2news@'. The url should be in the QuickSilver section of URLs.txt.    \n"
         "You can check this by opening QSstats and click 'Tools|URL Manager'.    \n"
         "QuickSilver is at the bottom.   \n\n"
         "Click Cancel to return to your original list.",
      MT_INFO, MB_OK);

   }

   OkBtn->Enabled = !boguspage;
   HttpBtn->Enabled = true;
   SetToolbarButtons();

   Editor->Lines->Text = page->Text;
   Editor->SelStart = 0;
   Editor->Perform(EM_SCROLLCARET, 0, 0);

   delete page;

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::HttpBtnClick(TObject *Sender)
{
   Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::CancelBtnClick(TObject *Sender)
{
   if (boguspage)
   {
      Editor->Clear();
      Snooze(200);
      Editor->Lines->AddStrings(OrgPage);
      Editor->Modified = OrgModified;
      OkBtn->Enabled = true;
      return;
   }

   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::CutBtnClick(TObject *Sender)
{
   Editor->CutToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::CopyBtnClick(TObject *Sender)
{
   Editor->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::PasteBtnClick(TObject *Sender)
{
   Editor->PasteFromClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TMail2newsDlg::FormShow(TObject *Sender)
{
   //TColor color;
   //LoadFonts(AppIni(), NymIni, color, Font, NULL, Editor->Font);
   //Editor->Color = color;
}
//---------------------------------------------------------------------------

