//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "NymAdvDlgs.h"
#include "NymRemEncChainDlgs.h"
#include "pgpsdk.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymAdvDlg *NymAdvDlg;
int const mNone = 0;
int const mExt = 1;
//---------------------------------------------------------------------------

bool __fastcall EditAdvancedRemailerDirectives(TForm *Owner, TTreeNode *Remailer,
   const AnsiString &Caps, bool Post, bool Last)
{
   bool ok = false;

   TStringList *spec = new TStringList();
   for (int i = 0; i < Remailer->Count; i++)
   {
      TTreeNode *node = Remailer->Item[i];
      spec->Add(node->Text);
   }

   if (EditAdvancedRemailerDirectives(Owner, spec, Caps, Post, Last))
   {
      Remailer->DeleteChildren();
      Snooze(300);

      TTreeView *TreeView = (TTreeView*)Remailer->TreeView;
      for (int i = 0; i < spec->Count; i++)
      {
         AnsiString line = spec->Strings[i].Trim();
         if (line.IsEmpty())
            continue;

         TreeView->Items->AddChild(Remailer, line);
      }

      Remailer->MakeVisible();
      Remailer->Item[Remailer->Count - 1]->MakeVisible();
      ok = true;
   }

   delete spec;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall EditAdvancedRemailerDirectives(TForm *Owner, TStrings *Spec,
   const AnsiString &Caps, bool Post, bool Last)
{
   NymAdvDlg = new TNymAdvDlg(Owner, Spec, Caps, Post, Last);

   NymAdvDlg->ShowModal();
   bool ok = NymAdvDlg->ModalResult == mrOk;

   delete NymAdvDlg;

   return ok;
}

//---------------------------------------------------------------------------
// NymAdvDlg
//---------------------------------------------------------------------------

__fastcall TNymAdvDlg::TNymAdvDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNymAdvDlg::TNymAdvDlg(TComponent* Owner, TStrings *Spec,
   const AnsiString &Caps, bool Post, bool Last) : TForm(Owner)
{
   spec = Spec;
   caps = Caps;
   post = Post;
   last = Last;

   Caption = Caption + " -- " + GetTok(1, caps, '\"');
   Caption = Caption + " <" + GetTok(0, GetTok(1, caps, '<'), '>') + ">";

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, AnonTo->Font, NULL);
   AnonTo->Color = color;

   // delivery directives
   AnonPostTo->Text = GetDirective("Anon-Post-To: ");
   AnonTo->Text = GetDirective("Anon-To: ");
   RemixTo->Text = GetDirective("Remix-To: ");
   EncryptTo->Text = GetDirective("Encrypt-To: ");
   RemailTo->Text = GetDirective("Remail-To: ");
   TestTo->Text = GetDirective("Test-To: ");
   SetDeliverToggles();

   AnonPostTo->Font->Assign(AnonTo->Font); AnonPostTo->Color = color;
   RemixTo->Font->Assign(AnonTo->Font); RemixTo->Color = color;
   EncryptTo->Font->Assign(AnonTo->Font); EncryptTo->Color = color;
   RemailTo->Font->Assign(AnonTo->Font); RemailTo->Color = color;
   TestTo->Font->Assign(AnonTo->Font); TestTo->Color = color;

   // encrypt
   EncryptKey->Text = GetDirective("Encrypt-Key: ");
   Encrypt3DES->Text = GetDirective("Encrypt-3DES: ");
   EncryptCAST->Text = GetDirective("Encrypt-CAST: ");
   EncryptSubject->Text = GetDirective("Encrypt-Subject: ");
   EncryptTest->Text = GetDirective("Encrypt-Test: ");
   SetEncryptToggles();

   EncryptKey->Font->Assign(AnonTo->Font); EncryptKey->Color = color;
   Encrypt3DES->Font->Assign(AnonTo->Font); Encrypt3DES->Color = color;
   EncryptCAST->Font->Assign(AnonTo->Font); EncryptCAST->Color = color;
   EncryptSubject->Font->Assign(AnonTo->Font); EncryptSubject->Color = color;
   EncryptTest->Font->Assign(AnonTo->Font); EncryptTest->Color = color;

   // final headers
   Subject->Text = GetDirective("Subject: ");
   Newsgroups->Text = GetDirective("Newsgroups: ");
   FollowupTo->Text = GetDirective("Followup-To: ");
   From->Text = GetDirective("From: ");
   CC->Text = GetDirective("CC: ");
   Bcc->Text = GetDirective("Bcc: ");
   ReplyTo->Text = GetDirective("Reply-To: ");
   SetHeaderToggles();

   Subject->Font->Assign(AnonTo->Font); Subject->Color = color;
   Newsgroups->Font->Assign(AnonTo->Font); Newsgroups->Color = color;
   FollowupTo->Font->Assign(AnonTo->Font); FollowupTo->Color = color;
   From->Font->Assign(AnonTo->Font); From->Color = color;
   CC->Font->Assign(AnonTo->Font); CC->Color = color;
   Bcc->Font->Assign(AnonTo->Font); Bcc->Color = color;
   ReplyTo->Font->Assign(AnonTo->Font); ReplyTo->Color = color;

   // miscellaneous
   LatentTime->Text = GetDirective("Latent-Time: ");
   Inflate->Text  = GetDirective("Inflate: ");
   RandHop->Text  = GetDirective("Rand-Hop: ");
   Cutmarks->Text = GetDirective("Cutmarks: ");
   MaxSize->Text  = GetDirective("Max-Size: ");
   MaxCount->Text = GetDirective("Max-Count: ");
   MaxDate->Text  = GetDirective("Max-Date: ");
   SetMiscToggles();

   LatentTime->Font->Assign(AnonTo->Font); LatentTime->Color = color;
   Inflate->Font->Assign(AnonTo->Font); Inflate->Color = color;
   RandHop->Font->Assign(AnonTo->Font); RandHop->Color = color;
   Cutmarks->Font->Assign(AnonTo->Font); Cutmarks->Color = color;
   MaxSize->Font->Assign(AnonTo->Font); MaxSize->Color = color;
   MaxCount->Font->Assign(AnonTo->Font); MaxCount->Color = color;
   MaxDate->Font->Assign(AnonTo->Font); MaxDate->Color = color;

   TDateTime date = Date();
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date +=  7));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date +=  7));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date +=  7));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date +=  7));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date += 14));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date += 14));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date += 14));
   MaxDate->Items->Add(FormatDateTime("dd-mmm-yyyy", date += 14));

   // set not modified for future reference
   AnonPostTo->Modified = false;
   AnonTo->Modified = false;
   RemixTo->Modified = false;
   EncryptTo->Modified = false;
   RemailTo->Modified = false;
   TestTo->Modified = false;

   Subject->Modified = false;
   Newsgroups->Modified = false;
   FollowupTo->Modified = false;
   From->Modified = false;
   CC->Modified = false;
   Bcc->Modified = false;
   ReplyTo->Modified = false;

   EncryptKey->Modified = false;
   Encrypt3DES->Modified = false;
   EncryptCAST->Modified = false;
   EncryptSubject->Modified = false;
   EncryptTest->Modified = false;

   orglatenttime = LatentTime->Text;
   orginflate = Inflate->Text;
   orgrandhop = RandHop->Text;
   orgcutmarks = Cutmarks->Text;
   orgmaxdate = MaxDate->Text;
   orgmaxsize = MaxSize->Text;
   orgmaxcount = MaxCount->Text;
}
//---------------------------------------------------------------------------

bool __fastcall TNymAdvDlg::Modified()
{
   bool ok =
      AnonPostTo->Modified ||
      AnonTo->Modified ||
      RemixTo->Modified ||
      EncryptTo->Modified ||
      RemailTo->Modified ||
      TestTo->Modified ||

      Subject->Modified ||
      Newsgroups->Modified ||
      FollowupTo->Modified ||
      From->Modified ||
      CC->Modified ||
      Bcc->Modified ||
      ReplyTo->Modified ||

      EncryptKey->Modified ||
      Encrypt3DES->Modified ||
      EncryptCAST->Modified ||
      EncryptSubject->Modified ||
      EncryptTest->Modified ||

      LatentTime->Text != orglatenttime ||
      Inflate->Text != orginflate ||
      RandHop->Text != orgrandhop ||
      Cutmarks->Text != orgcutmarks ||
      MaxDate->Text != orgmaxdate ||
      MaxSize->Text != orgmaxsize ||
      MaxCount->Text != orgmaxcount;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SaveToSpec()
{
   DelDirective("Test-To: ");
   if (!TestTo->Text.IsEmpty())
      spec->Insert(0, "Test-To: " + TestTo->Text);

   SetDirective("Test-To: ", TestTo->Text);
   SetDirective("Anon-Post-To: ", AnonPostTo->Text);

   if (ext)
   {
      if (AnonToBox->Checked)
         SetDirective("Anon-To: ", AnonTo->Text);
      else
         DelDirective("Anon-To: ");

      if (RemixToBox->Checked)
         SetDirective("Remix-To: ", RemixTo->Text);
      else
         DelDirective("Remix-To: ");

      if (EncryptToBox->Checked)
         SetDirective("Encrypt-To: ", EncryptTo->Text);
      else
         DelDirective("Encrypt-To: ");

      if (RemailToBox->Checked)
         SetDirective("Remail-To: ", RemailTo->Text);
      else
         DelDirective("Remail-To: ");
   }
   else
   {
      if (AnonToBtn->Checked)
         SetDirective("Anon-To: ", AnonTo->Text);
      else
         DelDirective("Anon-To: ");

      if (RemixToBtn->Checked)
         SetDirective("Remix-To: ", RemixTo->Text);
      else
         DelDirective("Remix-To: ");

      if (EncryptToBtn->Checked)
         SetDirective("Encrypt-To: ", EncryptTo->Text);
      else
         DelDirective("Encrypt-To: ");

      if (RemailToBtn->Checked)
         SetDirective("Remail-To: ", RemailTo->Text);
      else
         DelDirective("Remail-To: ");
   }

   SetDirective("Subject: ", Subject->Text);
   SetDirective("Newsgroups: ", Newsgroups->Text);
   SetDirective("Followup-To: ", FollowupTo->Text);
   SetDirective("From: ", From->Text);
   SetDirective("CC: ", CC->Text);
   SetDirective("Bcc: ", Bcc->Text);
   SetDirective("Reply-To: ", ReplyTo->Text);

   SetDirective("Encrypt-Key: ", EncryptKey->Text);
   SetDirective("Encrypt-3DES: ", Encrypt3DES->Text);
   SetDirective("Encrypt-CAST: ", EncryptCAST->Text);
   SetDirective("Encrypt-Subject: ", EncryptSubject->Text);
   SetDirective("Encrypt-Test: ", EncryptTest->Text);

   SetDirective("Latent-Time: ", LatentTime->Text);
   SetDirective("Inflate: ", Inflate->Text);
   SetDirective("Rand-Hop: ", RandHop->Text);
   SetDirective("Cutmarks: ", Cutmarks->Text);
   SetDirective("Max-Date: ", MaxDate->Text);
   SetDirective("Max-Size: ", MaxSize->Text);
   SetDirective("Max-Count: ", MaxCount->Text);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymAdvDlg::GetDirective(const AnsiString &Directive)
{
   AnsiString line, value;

   for (int i = 0; i < spec->Count; i++)
   {
      line = spec->Strings[i];
      if (line.Pos(Directive) == 1)
      {
         value = line.SubString(line.Pos(":") + 1, 128).Trim();
         break;
      }
   }
   return value;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetDirective(const AnsiString &Directive, const AnsiString &Value)
{
   AnsiString directive = Directive.TrimRight() + " " + Value;
   int idx = DelDirective(Directive);
   if (Value.IsEmpty())
      return;

   if (idx >= 0) spec->Insert(idx, directive);
   else spec->Add(directive);
}
//---------------------------------------------------------------------------

int __fastcall TNymAdvDlg::DelDirective(const AnsiString &Directive)
{
   int idx = -1;
   for (int i = 0; i < spec->Count; i++)
   {
      if (spec->Strings[i].Pos(Directive) == 1)
      {
         spec->Delete(i);
         idx = i;
         break;
      }
   }
   return idx;
}
//---------------------------------------------------------------------------

bool __fastcall TNymAdvDlg::HasCap(const AnsiString &Cap)
{
   return caps.Pos(" " + Cap + " ") > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::EnableControl(TEdit *Control, bool Enable)
{
   Control->Color = Enable ? clWindow : clBtnFace;
   Control->Enabled = Enable;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::DisableControl(TComboBox *Control, TLabel *Label)
{
   Control->ParentColor = true;
   Control->Enabled = false;
   Label->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::DisableControl(TEdit *Control, TLabel *Label)
{
   Control->ParentColor = true;
   Control->Enabled = false;
   Label->Enabled = false;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymAdvDlg::GetAnonToAddress()
{
   AnsiString address, tmp = AnonTo->Text;
   if (!tmp.IsEmpty())
      return tmp;

   if (tmp.IsEmpty())
      tmp = RemixTo->Text;
   if (tmp.IsEmpty())
      tmp = EncryptTo->Text;
   if (tmp.IsEmpty())
      tmp = RemailTo->Text;

   for (int i = tmp.Length(); i > 0; i--)
   {
      if (tmp[i] == ',' || tmp[i] == ' ')
         break;

      address = AnsiString(tmp[i]) + address;
   }

   return address;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::DisableDelivery(TCheckBox *Box, TRadioButton *Btn,
   TEdit *Editor)
{
   Box->Enabled = false;
   Btn->Enabled = false;
   Editor->Enabled = false;
   Editor->ParentColor = true;
   Editor->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetExtMode()
{
   AnonToBtn->Visible = false;
   AnonToBox->Visible = true;

   RemixToBtn->Visible = false;
   RemixToBox->Visible = true;
   RemixToBox->Enabled = true;
   RemixTo->Width = orgremixtowidth;

   EncryptToBtn->Visible = false;
   EncryptToBox->Visible = true;
   EncryptToBox->Enabled = true;
   EncryptTo->Width = orgremixtowidth;

   RemailToBtn->Visible = false;
   RemailToBox->Visible = true;

   if (!AnonTo->Text.IsEmpty())
      AnonToBox->Checked = true;

   if (HasCap("remix") && !RemixTo->Text.IsEmpty())
      RemixToBox->Checked = true;

   if (HasCap("repgp") && !EncryptTo->Text.IsEmpty())
      EncryptToBox->Checked = true;

   if (!RemailTo->Text.IsEmpty())
      RemailToBox->Checked = true;

   if (!AnonToBox->Checked)
      EnableControl(AnonTo, false);

   if (!RemixToBox->Checked)
   {
      EnableControl(RemixTo, false);
      RemixToChainBtn->Enabled = false;
      RemixToChainBtn->Visible = true;
   }

   if (!EncryptToBox->Checked)
   {
      EnableControl(EncryptTo, false);
      EncryptToChainBtn->Enabled = false;
      EncryptToChainBtn->Visible = true;
   }

   if (!RemailToBox->Checked)
      EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetDeliverToggles()
{
   AnonToBox->Left = AnonToBtn->Left;
   RemixToBox->Left = RemixToBtn->Left;
   EncryptToBox->Left = EncryptToBtn->Left;
   RemailToBox->Left = RemailToBtn->Left;
   AnonToBox->Visible = false;
   RemixToBox->Visible = false;
   EncryptToBox->Visible = false;
   RemailToBox->Visible = false;
   RemixToChainBtn->Visible = false;
   EncryptToChainBtn->Visible = false;
   orgremixtowidth = RemixTo->Width;
   EncryptTo->Width = AnonTo->Width;
   RemixTo->Width = AnonTo->Width;

   EncryptTest->ParentColor = true;
   EncryptTest->Enabled = false;
   Button5->Enabled = false;
   Label1->Enabled = false;

   ext =  HasCap("ext");
   if (ext)
      SetExtMode();

   if (!last || !post)
   {
      AnonPostTo->ParentColor = true;
      AnonPostTo->Enabled = false;
      Label21->Enabled = false;
   }

   if (!post)
   {
      AnonToBtnClick(NULL);
   }


   bool remix = HasCap("remix");
   if (last) // || !remix)
   {
      RemixToChainBtn->Enabled = false;
      RemixToBtn->Enabled = false;
      RemixToBox->Enabled = false;
      RemixTo->Enabled = false;
      RemixTo->ParentColor = true;
   }

   bool repgp = HasCap("repgp");
   if (last) // || !repgp)
   {
      EncryptToChainBtn->Enabled = false;
      EncryptToBtn->Enabled = false;
      EncryptToBox->Enabled = false;
      EncryptTo->Enabled = false;
      EncryptTo->ParentColor = true;
   }

   bool testto = HasCap("test");
   if (!testto)
   {
      TestTo->ParentColor = true;
      TestTo->Enabled = false;
      Label2->Enabled = false;
      EncryptTest->ParentColor = true;
      EncryptTest->Enabled = false;
      Label1->Enabled = false;
   }

   AnsiString addr = GetAnonToAddress();
   if (AnonTo->Text.IsEmpty())
      AnonTo->Text = addr;

   if (!last && remix && RemixTo->Text.IsEmpty())
      RemixTo->Text = addr;
   if (!last && repgp && EncryptTo->Text.IsEmpty())
      EncryptTo->Text = addr;

   if (RemailTo->Text.IsEmpty())
      RemailTo->Text = addr;

   if (last && post)
   {
      AnonToBtn->Checked = false;
      AnonToBox->Checked = false;
      DisableDelivery(AnonToBox, AnonToBtn, AnonTo);
      DisableDelivery(RemixToBox, RemixToBtn, RemixTo);
      DisableDelivery(EncryptToBox, EncryptToBtn, EncryptTo);
      DisableDelivery(RemailToBox, RemailToBtn, RemailTo);
      ActiveControl = AnonPostTo;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::AnonToBtnClick(TObject *)
{
   EnableControl(AnonTo, true);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::RemixToBtnClick(TObject *)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, true);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::EncryptToBtnClick(TObject *)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, true);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::RemailToBtnClick(TObject *)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, true);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::AnonToBoxClick(TObject *)
{
   EnableControl(AnonTo, AnonToBox->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::RemixToBoxClick(TObject *)
{
   bool checked = RemixToBox->Checked;
   RemixToChainBtn->Enabled = checked;
   EnableControl(RemixTo, checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::EncryptToBoxClick(TObject *)
{
   bool checked = EncryptToBox->Checked;
   EncryptToChainBtn->Enabled = checked;
   EnableControl(EncryptTo, checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::RemailToBoxClick(TObject *)
{
   EnableControl(RemailTo, RemailToBox->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::TestToExit(TObject *)
{
   if (!TestTo->Text.IsEmpty())
   {
      EnableControl(EncryptTest, true);
      Label1->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::TestToChange(TObject *)
{
   bool enable = !TestTo->Text.IsEmpty();
   if (enable)
   {
      EncryptTest->Enabled = true;
      EncryptTest->Color = clWindow;
      Button5->Enabled = true;
      Label1->Enabled = true;
      if (EncryptTest->Text.IsEmpty())
         EncryptTest->Text = PGP->GetEncryptKey(ekLength);
   }
   else
   {
      EncryptTest->Enabled = false;
      EncryptTest->ParentColor = true;
      Button5->Enabled = false;
      Label1->Enabled = false;
      EncryptTest->Text = "";
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::PasswordEnter(TObject *)
{
   if (EncryptTest->Text.IsEmpty())
      EncryptTest->Text = PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::RemixToChainBtnClick(TObject *)
{
   AnsiString remixto = RemixTo->Text;
   if (EditRemixEncryptChain(this, "Select Remix-To Remailer Chain", remixto))
   {
      RemixTo->Text = remixto;
      RemixTo->Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::EncryptToChainBtnClick(TObject *)
{
   AnsiString encryptto = EncryptTo->Text;
   if (EditRemixEncryptChain(this, "Select Encrypt-To Remailer Chain", encryptto))
   {
      EncryptTo->Text = encryptto;
      EncryptTo->Modified = true;
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymAdvDlg::ValidAddress(AnsiString Address)
{
   if (Address.IsEmpty() || (ext && Address.LowerCase() == "me"))
      return true;

   int at = 0; for (int i = 0; i < Address.Length(); i++)
      if (Address[i] == '@')
         at++;

   if (at > 1)
   {
      MsgBox((TForm*)Owner,
         "Your Test-To address is not a valid email address!   ",
          MT_INFO, MB_OK);
      return false;
   }

   at = Address.Pos("@");
   int pt = Address.SubString(at + 1, 256).Pos(".");
   if (at == 0 || pt == 0)
   {
      MsgBox((TForm*)Owner, "Your Test-To address is not a valid email address!   ",
         MT_INFO, MB_OK);
      return false;
   }

   return true;
}

//---------------------------------------------------------------------------
// ENCRYPT DIRECTIVES
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetEncryptToggles()
{
   if (!HasCap("ek"))
   {
      //KEY->Checked = false;
      //KEY->Enabled = false;
      EncryptKey->ParentColor = true;
      EncryptKey->Enabled = false;
      EncryptKey->Text = "";
      Label3->Enabled = false;
   }

   if (!HasCap("ekx"))
   {
      //DES->Checked = false;
      //DES->Enabled = false;
      Encrypt3DES->ParentColor = true;
      Encrypt3DES->Enabled = false;
      Encrypt3DES->Text = "";
      Label4->Enabled = false;

      //CAS->Checked = false;
      //CAS->Enabled = false;
      EncryptCAST->ParentColor = true;
      EncryptCAST->Enabled = false;
      EncryptCAST->Text = "";
      Label5->Enabled = false;
   }
}
//---------------------------------------------------------------------------

/*void __fastcall TNymAdvDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (KEY->Checked && KEYPassword->Text.IsEmpty())
      {
         MsgDlg(this, "Encrypt-Key is checked. You must enter a password.",
            "Error", MB_OK);
         CanClose = false;
         return;
      }
      if (DES->Checked && DESPassword->Text.IsEmpty())
      {
         MsgDlg(this, "Encrypt-3DES is checked. You must enter a password.",
            "Error", MB_OK);
         CanClose = false;
         return;
      }
      if (CAS->Checked && CASPassword->Text.IsEmpty())
      {
         MsgDlg(this, "Encrypt-CAST is checked. You must enter a password.",
            "Error", MB_OK);
         CanClose = false;
         return;
      }
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::KEYPasswordEnter(TObject *)
{
   if (EncryptKey->Text.IsEmpty())
   {
      EncryptKey->Text = PGP->GetEncryptKey(ekLength);
      EncryptKey->SelectAll();
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::DESPasswordEnter(TObject *)
{
   if (Encrypt3DES->Text.IsEmpty())
   {
      Encrypt3DES->Text = PGP->GetEncryptKey(ekLength);
      Encrypt3DES->SelectAll();
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::CASPasswordEnter(TObject *)
{
   if (EncryptCAST->Text.IsEmpty())
   {
      EncryptCAST->Text = PGP->GetEncryptKey(ekLength);
      EncryptCAST->SelectAll();
   }
}

//---------------------------------------------------------------------------
// Headers
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetHeaderToggles()
{
   if (!last)
   {
      DisableControl(Subject, Label14);
      DisableControl(Newsgroups, Label15);
      DisableControl(FollowupTo, Label16);
      DisableControl(From, Label17);
      DisableControl(CC, Label18);
      DisableControl(Bcc, Label19);
      DisableControl(ReplyTo, Label20);
   }
}

//---------------------------------------------------------------------------
// Miscellaneous
//---------------------------------------------------------------------------

int __fastcall TNymAdvDlg::GetCapMax(const AnsiString &Cap)
{
   int val = 0;
   AnsiString cap;

   int i = 0; while ((cap = GetTok(i, caps, ' ')) != "")
   {
      if (cap.Pos(Cap) == 1)
      {
         val = cap.SubString(Cap.Length() + 1, 4).ToIntDef(0);
         break;
      }
      i++;
   }

   return val;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::SetMiscToggles()
{
   if (!HasCap("inflt")) DisableControl(Inflate, Label9);
   if (!HasCap("rhop"))  DisableControl(RandHop, Label10);
   if (!HasCap("cut"))   DisableControl(Cutmarks, Label11);
   if (!HasCap("max"))
   {
      DisableControl(MaxCount, Label7);
      DisableControl(MaxDate, Label8);
      DisableControl(MaxSize, Label6);
   }

   if (HasCap("inflt"))
   {
      AnsiString max = IntToStr(GetCapMax("inflt")) + "r", hold = max;
      while (max.Length() < 4)
         max = "0" + max;

      while (Inflate->Items->Count > 0)
      {
         int c = Inflate->Items->Count;
         AnsiString val = Inflate->Items->Strings[c - 1];
         while (val.Length() < 4)
            val = "0" + val;

         if (val >= max)
            Inflate->Items->Delete(Inflate->Items->Count - 1);
         else
            break;
      }

      Inflate->Items->Add(hold);
   }

   if (HasCap("rhop"))
   {
      RandHop->Clear();
      int max = GetCapMax("rhop");
      if (max == 0) max = 10;

      for (int i = 0; i < max; i++)
      {
         RandHop->Items->Add(IntToStr(i) + "r");
      }

      for (int i = 0; i < max; i++)
      {
         RandHop->Items->Add(IntToStr(i));
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::FormCloseQuery(TObject *, bool &CanClose)
{
   if (Modified())
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this,
            "Your remailer configuration is modified.    \n\n"
            "Save your changes?", MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }
      SaveToSpec();
      return;
   }
   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::Button1Click(TObject *)
{
   EncryptKey->Text = EncryptKey->Text + PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::Button2Click(TObject *)
{
   Encrypt3DES->Text = Encrypt3DES->Text + PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::Button3Click(TObject *)
{
   EncryptCAST->Text = EncryptCAST->Text + PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::Button4Click(TObject *)
{
   EncryptSubject->Text = EncryptSubject->Text + PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------

void __fastcall TNymAdvDlg::Button5Click(TObject *)
{
   EncryptTest->Text = EncryptTest->Text + PGP->GetEncryptKey(ekLength);
}
//---------------------------------------------------------------------------
