//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "NymOptionsDlgs.h"
#include "NymDlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymOptionsDlg *NymOptionsDlg;
//---------------------------------------------------------------------------

bool __fastcall OpenNymOptionsDialog(TForm *Owner, TNymsFrame2 *Frame, const AnsiString &Title)
{
   if (SecureModeMsg(Owner))
      return false;

   NymOptionsDlg = new TNymOptionsDlg(Owner, Title);
   NymOptionsDlg->Font->Assign(Owner->Font);
   NymOptionsDlg->ShowModal();
   if (NymOptionsDlg->ModalResult == mrOk)
   {
      Frame->Color = NymOptionsDlg->WorkColor->Color;
      StrToFont(NymOptionsDlg->WinsFontStr, Frame->Font);
      StrToFont(NymOptionsDlg->DlgsFontStr, Owner->Font);
   }
   bool ok = NymOptionsDlg->ModalResult == mrOk;
   delete NymOptionsDlg;
   return ok;
}

//---------------------------------------------------------------------------
// TNymOptionsDlg
//---------------------------------------------------------------------------

__fastcall TNymOptionsDlg::TNymOptionsDlg(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNymOptionsDlg::TNymOptionsDlg(TComponent* Owner, const AnsiString &Title)
   : TForm(Owner)
{
   Caption = Title;
   IniModified = false;
   PageControl->ActivePage = GenTab;

   TIniFile *ini = new TIniFile(AppIni());

   WorkColor->Color = (TColor)ini->ReadInteger(NymIni, QSWINDOWCOLOR, QSWINDOWCOLORDEF);
   WinsFontStr = ini->ReadString(NymIni, QSWINDOWFONT, QSWINDOWFONTDEF);
   DlgsFontStr = ini->ReadString(NymIni, QSDIALOGFONT, QSDIALOGFONTDEF);
   EditFontStr = ini->ReadString(NymIni, QSEDITORFONT, QSEDITORFONTDEF);

   StrToFontBox(WinsFontStr, Name1, Size1, Style1, Color1);
   StrToFontBox(DlgsFontStr, Name2, Size2, Style2, Color2);
   StrToFontBox(EditFontStr, Name6, Size6, Style6, Color6);

   NymServers->Color = WorkColor->Color;
   StrToFont(WinsFontStr, NymServers->Font);
   LatentBox->Color = WorkColor->Color;
   StrToFont(WinsFontStr, LatentBox->Font);

   OrgOptions = Global->DB->GetIni("defnymoptions",
      "+acksend,-signsend,+cryptrecv,-disable,-fixedsize,-fingerkey,+nobcc");

   OptAck->Checked = OrgOptions.Pos("+acksend") > 0;
   OptSign->Checked = OrgOptions.Pos("+signsend") > 0;
   OptCrypt->Checked = OrgOptions.Pos("+cryptrecv") > 0;
   OptDisable->Checked = OrgOptions.Pos("+disable") > 0;
   OptFixed->Checked = false; //OrgOptions.Pos("+fixedsize") > 0;
   OptFixed->Enabled = false;
   OptFinger->Checked = OrgOptions.Pos("+fingerkey") > 0;
   OptBcc->Checked = OrgOptions.Pos("+nobcc") > 0;

   GetNymServers(NymServers->Items);
   OrgNymServer = Global->DB->GetIni("defnymserver", "hod.aarg.net");
   NymServers->ItemIndex = NymServers->Items->IndexOf(OrgNymServer);

   OrgLatentTime = Global->DB->GetIni("deflatenttime", "+1:00r");
   LatentBox->Text = OrgLatentTime;

   delete ini;
}
//---------------------------------------------------------------------------

__fastcall TNymOptionsDlg::~TNymOptionsDlg()
{
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TNymOptionsDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   AnsiString options;
   options += OptAck->Checked ? "+acksend" : "-acksend";
   options += OptSign->Checked ? " +signsend" : " -signsend";
   options += OptCrypt->Checked ? " +cryptrecv" : " -cryptrecv";
   options += OptDisable->Checked ? " +disable" : " -disable";
   options += OptFixed->Checked ? " +fixedsize" : " -fixedsize";
   options += OptFinger->Checked ? " +fingerkey" : " -fingerkey";
   options += OptBcc->Checked ? " +nobcc" : " -nobcc";
   if (options != OrgOptions)
      IniModified = true;

   if (IniModified && SecureModeMsg(this))
   {
      CanClose = false;
      return;
   }

   if (NymServers->ItemIndex < 0)
      NymServers->ItemIndex = NymServers->Items->IndexOf("hod.aarg.net");

   if (NymServers->ItemIndex < 0)
      NymServers->ItemIndex = 0;

   AnsiString nymserver = NymServers->Items->Strings[NymServers->ItemIndex];
   if (nymserver != OrgNymServer)
      IniModified = true;

   AnsiString latenttime = LatentBox->Text;
   if (latenttime.IsEmpty())
   {
      if (MsgBox(this,
         "You have left the Latent-Time default empty. When this is the case, reply    \n"
         "blocks will not have a Latent-time directive added.\n\n"
         "Continue with Latent-Time empty?",
          MT_INFO, MB_OKCANCEL) == IDCANCEL)
      {
         CanClose = false;
         return;
      }
   }
   if (!latenttime.IsEmpty() && latenttime != OrgLatentTime && LatentBox->Items->IndexOf(latenttime) < 0)
   {
      if (MsgBox(this,
         "I see you have entered a value for Latent-Time that is not listed in the    \n"
         "Latent-Time dropdown list. You can do this but there are many valid choices    \n"
         "to pick from. I assume you know what you are doing and so your value will    \n"
         "not be checked for errors.\n\nContinue with your Latent-Time as it is now?",
          MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
      {
         CanClose = false;
         return;
      }
   }
   if (latenttime != OrgLatentTime)
      IniModified = true;

   if (IniModified)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this, "Your options are modified. Save your changes?",
            MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      ModalResult = mrOk;
      TIniFile *ini = new TIniFile(AppIni());

      ini->WriteInteger(NymIni, QSWINDOWCOLOR, WorkColor->Color);
      ini->WriteString(NymIni, QSWINDOWFONT, WinsFontStr);
      ini->WriteString(NymIni, QSDIALOGFONT, DlgsFontStr);
      ini->WriteString(NymIni, QSEDITORFONT, EditFontStr);

      delete ini;

      Global->DB->PutIni("defnymserver", nymserver);
      Global->DB->PutIni("defnymoptions", options);
      Global->DB->PutIni("deflatenttime", latenttime);
   }
}

//---------------------------------------------------------------------------
// General
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::WorkColorBtnClick(TObject *)
{
   ColorDialog->Color = WorkColor->Color;
   if (ColorDialog->Execute())
   {
      TColor color = ColorDialog->Color;
      WorkColor->Color = color;
      NymServers->Color = color;

      IniModified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::StrToFontBox(const AnsiString &FontStr, TEdit *Name,
   TEdit *Size, TEdit *Style, TPanel *Color)
{
   Name->Text  = GetTok(0, FontStr, ',');
   Size->Text  = GetTok(1, FontStr, ',');
   Style->Text = GetTok(2, FontStr, ',');
   Color->Color = (TColor)GetTok(3, FontStr, ',').ToIntDef(0);
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::StrToFont(const AnsiString &FontStr, TFont *Font)
{
   Font->Name = GetTok(0, FontStr, ',');
   Font->Size = GetTok(1, FontStr, ',').ToIntDef(8);

   Font->Style.Clear();
   AnsiString stylestr = GetTok(2, FontStr, ',').LowerCase();
   if (stylestr.Pos("b") > 0) Font->Style << fsBold;
   if (stylestr.Pos("i") > 0) Font->Style << fsItalic;
   if (stylestr.Pos("u") > 0) Font->Style << fsUnderline;
   if (stylestr.Pos("s") > 0) Font->Style << fsStrikeOut;

   Font->Color = (TColor)GetTok(3, FontStr, ',').ToIntDef(0);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymOptionsDlg::FontToStr(TFont *Font)
{
   AnsiString fontstr = Font->Name;
   fontstr += "," + IntToStr(Font->Size);

   fontstr += ",";
   if (Font->Style.Contains(fsBold)) fontstr += "b";
   if (Font->Style.Contains(fsItalic)) fontstr += "i";
   if (Font->Style.Contains(fsUnderline)) fontstr += "u";
   if (Font->Style.Contains(fsStrikeOut)) fontstr += "s";

   fontstr += "," + IntToStr(Font->Color);
   return fontstr;
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::WinFontBtnClick(TObject *)
{
   StrToFont(WinsFontStr, FontDialog->Font);
   if (FontDialog->Execute())
   {
      WinsFontStr = FontToStr(FontDialog->Font);
      StrToFontBox(WinsFontStr, Name1, Size1, Style1, Color1);
      NymServers->Font->Assign(FontDialog->Font);
      LatentBox->Font->Assign(FontDialog->Font);

      IniModified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::DlgFontBtnClick(TObject *)
{
   StrToFont(DlgsFontStr, FontDialog->Font);
   if (FontDialog->Execute())
   {
      DlgsFontStr = FontToStr(FontDialog->Font);
      StrToFontBox(DlgsFontStr, Name2, Size2, Style2, Color2);
      Font->Assign(FontDialog->Font);

      IniModified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::EditFontBtnClick(TObject *)
{
   StrToFont(EditFontStr, FontDialog->Font);
   if (FontDialog->Execute())
   {
      EditFontStr = FontToStr(FontDialog->Font);
      StrToFontBox(EditFontStr, Name6, Size6, Style6, Color6);

      IniModified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymOptionsDlg::NymServersClick(TObject *)
{
   if (NymServers->Items->Strings[NymServers->ItemIndex] != OrgNymServer)
      IniModified = true;
}
//---------------------------------------------------------------------------

